package com.dayou.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.dayou.common.BaseEntity;
import com.dayou.dto.OrderReportDTO;
import com.dayou.dto.ReportDTO;
import com.dayou.dto.WareHouseDTO;
import com.dayou.entity.*;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.enums.UserTypeEnum;
import com.dayou.mapper.MajorProductionMapper;
import com.dayou.mapper.MessageMapper;
import com.dayou.message.MessageTypeEnum;
import com.dayou.message.bo.BusinessMessageBO;
import com.dayou.message.bo.SystemMessageBO;
import com.dayou.message.handler.BusinessMessageHandler;
import com.dayou.message.handler.SystemMessageHandler;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.utils.LoginContext;
import com.dayou.vo.CommissionDeclareVO;
import com.dayou.vo.ItemVO;
import com.dayou.vo.MajorProductionVO;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

import static com.dayou.enums.BusinessReplyEnum.MARKET_LOG_REPLY;
import static com.dayou.enums.BusinessReplyEnum.MARKET_VISIT_REPLY;
import static com.dayou.message.MessageTitle.*;
import static com.dayou.message.MessageTitle.MAJOR_DECLARE_REMIND;
import static com.dayou.message.MessageTypeEnum.*;

/**
 * <p>
 * 消息 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-06-30
 */
@Service
public class MessageServiceImpl extends ServiceImpl<MessageMapper, Message> implements IMessageService {

    @Autowired
    private BusinessMessageHandler businessMessageHandler;

    @Autowired
    private SystemMessageHandler systemMessageHandler;

    @Autowired
    private ICommissionDeclareService commissionDeclareService;

    @Autowired
    private IMajorService majorService;

    @Autowired
    private IMajorProductionService majorProductionService;

    @Autowired
    private IUserService userService;

    @Autowired
    private MajorProductionMapper majorProductionMapper;

    @Autowired
    private IMajorProductionAllotService majorProductionAllotService;

    @Override
    public Boolean packagingBusinessMessage(BusinessReply originalBO) {
        BusinessMessageBO messageBO = new BusinessMessageBO();
        messageBO.setReceiver(originalBO.getReceiverId());
        String bizType = originalBO.getBizType().equals(MARKET_VISIT_REPLY.name())?MARKET_VISIT_REPLY.getName(): MARKET_LOG_REPLY.getName();
        messageBO.setTitle(BUSINESS_MESSAGE_TITLE.replace("x", bizType));
        messageBO.setMessage(originalBO.getContent());
        messageBO.setParams(JSON.toJSONString(originalBO));
        messageBO.setSender(originalBO.getReplierId());
        messageBO.setType(BUSINESS.name());
        return businessMessageHandler.sendMessage(messageBO);
    }

    @Override
    public Boolean packagingSettleMessage(Set<Long> userIds) {
        Set<SystemMessageBO> settleMessages = userIds.stream().map(x -> {
            SystemMessageBO systemMessageBO = new SystemMessageBO();
            systemMessageBO.setTitle(SYSTEM_MESSAGE_TITLE.replace("x", SETTLE.getName()));
            systemMessageBO.setMessage(SETTLE_MESSAGE_CONTENT);
            systemMessageBO.setReceiver(x);
            systemMessageBO.setType(SETTLE.name());
            return systemMessageBO;
        }).collect(Collectors.toSet());
        if (CollectionUtils.isNotEmpty(settleMessages)){
            return systemMessageHandler.sendMessage(settleMessages);
        }
        return Boolean.FALSE;
    }

    @Override
    public Boolean packagingMajorDeclareRemind(Long declareId) {
        BusinessMessageBO messageBO = new BusinessMessageBO();
        CommissionDeclareVO declareVO = new CommissionDeclareVO();
        CommissionDeclare originDeclare = commissionDeclareService.getById(declareId);
        String businessType = originDeclare.getBusinessType();


        String template = MAJOR_DECLARE_REMIND_CONTENT;
        declareVO.setId(originDeclare.getId());

        Major order = majorService.getOne(new LambdaQueryWrapper<Major>().select(Major::getClientManagerId, Major::getPrincipalId, Major::getOrderId, Major::getName)
                .eq(BaseEntity::getId, originDeclare.getBusinessId()));
        declareVO.setOrderName(order.getName());
        if (originDeclare.getProductionId()!=null){
            MajorProduction production = majorProductionService.getOne(new LambdaQueryWrapper<MajorProduction>().select(MajorProduction::getReportNo, MajorProduction::getName)
                    .eq(BaseEntity::getId, originDeclare.getProductionId()));
            template = template.replace("name",production.getName());
            declareVO.setName(production.getName());
            declareVO.setReportNo(production.getReportNo());
        }else {
            template = template.replace("name",order.getName());
        }
        messageBO.setType(MessageTypeEnum.MAJOR_DECLARE_REMIND.name());
        messageBO.setTitle(MAJOR_DECLARE_REMIND);
        messageBO.setParams(JSON.toJSONString(declareVO));
        if (MainBusinessEnum.COMMISSION_DECLARE_MAJOR_EVALUATE.name().equals(businessType)){
            //评估人员向市场人员发 已申报情况不发送消息
            int count = commissionDeclareService.count(new LambdaQueryWrapper<CommissionDeclare>()
                    .eq(CommissionDeclare::getBusinessType, MainBusinessEnum.COMMISSION_DECLARE_MAJOR_MARKET.name())
                    .eq(CommissionDeclare::getBusinessId, originDeclare.getBusinessId())
                    .eq(originDeclare.getProductionId()!=null,CommissionDeclare::getProductionId,originDeclare.getProductionId()));

            if (count>0){
                return Boolean.FALSE;
            }
            messageBO.setSender(originDeclare.getDeclareUserId());
            messageBO.setReceiver(order.getClientManagerId());
            String marketer = userService.getOne(new LambdaQueryWrapper<User>().select(User::getName).eq(BaseEntity::getId, order.getClientManagerId())).getName();
            template  = marketer + template;
            messageBO.setMessage(template);
            declareVO.setBusinessType(MainBusinessEnum.COMMISSION_DECLARE_MAJOR_MARKET.name());

        }
        if (MainBusinessEnum.COMMISSION_DECLARE_MAJOR_MARKET.name().equals(businessType)){
            //评估人员向评估人员发
            int count = commissionDeclareService.count(new LambdaQueryWrapper<CommissionDeclare>()
                    .eq(CommissionDeclare::getBusinessType, MainBusinessEnum.COMMISSION_DECLARE_MAJOR_EVALUATE.name())
                    .eq(CommissionDeclare::getBusinessId, originDeclare.getBusinessId())
                    .eq(originDeclare.getProductionId()!=null,CommissionDeclare::getProductionId,originDeclare.getProductionId()));
            if (count>0){
                return Boolean.FALSE;
            }
            messageBO.setSender(originDeclare.getDeclareUserId());
            if (order.getPrincipalId()==null){
                return Boolean.FALSE;
            }
            messageBO.setReceiver(order.getPrincipalId());
            String evaluator = userService.getOne(new LambdaQueryWrapper<User>().select(User::getName).eq(BaseEntity::getId, order.getPrincipalId())).getName();
            template  = evaluator + template;
            messageBO.setMessage(template);
            declareVO.setBusinessType(MainBusinessEnum.COMMISSION_DECLARE_MAJOR_EVALUATE.name());
        }
        messageBO.setParams(JSON.toJSONString(declareVO));
        return businessMessageHandler.sendMessage(messageBO);
    }

    @Override
    public List<Message> messageRedPoint() {
        Long currentUserId = LoginContext.getCurrentUserId();
        List<Message> list = this.list(new LambdaQueryWrapper<Message>().eq(Message::getReceiver, currentUserId)
                .eq(Message::getIsRead, Boolean.FALSE).orderByDesc(BaseEntity::getCreated));
        return list;
    }

    @Override
    public Boolean updateRead(Long id) {
        return this.update(new LambdaUpdateWrapper<Message>().eq(BaseEntity::getId,id).set(Message::getIsRead,Boolean.TRUE));
    }

    @Override
    public void packagingEvaluateRemind(List<ItemVO> completedNotEvaluateItem) {
        Set<SystemMessageBO> collect = completedNotEvaluateItem.stream().map(x -> {
            SystemMessageBO systemMessageBO = new SystemMessageBO();
            systemMessageBO.setType(EVALUATE_REMIND.name());
            systemMessageBO.setTitle(SYSTEM_MESSAGE_TITLE.replace("x", EVALUATE_REMIND.getName()));
            systemMessageBO.setMessage(EVALUATE_REMIND_CONTENT.replace("x", x.getName()));
            systemMessageBO.setParams(JSON.toJSONString(x));
            systemMessageBO.setReceiver(x.getUserId());
            return systemMessageBO;
        }).collect(Collectors.toSet());
        systemMessageHandler.sendMessage(collect);
    }

    @Override
    public Boolean allRead() {
        Long currentUserId = LoginContext.getCurrentUserId();
        return this.update(new LambdaUpdateWrapper<Message>().eq(Message::getReceiver,currentUserId).set(Message::getIsRead,true));
    }

    @Override
    public void packagingMajorOutRepertoryRemind(WareHouseDTO wareHouseDTO) {
        MajorProductionVO majorProductionVO = majorProductionMapper.getMajorProductionById(wareHouseDTO.getProductionId());
        SystemMessageBO systemMessageBO = new SystemMessageBO();
        systemMessageBO.setReceiver(majorProductionVO.getClientManagerId());
        systemMessageBO.setMessage(MAJOR_PRODUCTION_OUT_CONTENT.replace("reportNo",majorProductionVO.getReportNo()));
        systemMessageBO.setTitle(MAJOR_OUT_REPERTORY_REMIND.getName());
        systemMessageBO.setType(MAJOR_OUT_REPERTORY_REMIND.name());
        systemMessageBO.setParams(JSON.toJSONString(majorProductionVO));
        systemMessageHandler.sendMessage(systemMessageBO);
    }

    @Override
    public void packagingMajorAllotCommissionRemind(List<OrderReportDTO> list) {
        for (OrderReportDTO orderReportDTO : list){
            List<MajorProductionAllot> allots = majorProductionAllotService.list(new LambdaQueryWrapper<MajorProductionAllot>().eq(MajorProductionAllot::getMajorProductionId, orderReportDTO.getId())
                    .eq(MajorProductionAllot::getUserType, UserTypeEnum.EVALUATE.name()).eq(BaseEntity::getDeleted, Boolean.FALSE));
            if (CollectionUtil.isEmpty(allots)){
                MajorProductionVO majorProductionVO = majorProductionMapper.getMajorProductionById(orderReportDTO.getId());
                SystemMessageBO systemMessageBO = new SystemMessageBO();
                systemMessageBO.setReceiver(majorProductionVO.getDepartmentLeaderId());
                systemMessageBO.setMessage(MAJOR_ALLOT_COMMISSION_CONTENT.replace("reportNo",majorProductionVO.getReportNo()));
                systemMessageBO.setTitle(MAJOR_ALLOT_COMMISSION_REMIND1.getName());
                systemMessageBO.setType(MAJOR_ALLOT_COMMISSION_REMIND1.name());
                systemMessageBO.setParams(JSON.toJSONString(majorProductionVO));
                systemMessageHandler.sendMessage(systemMessageBO);
            }
        }

    }

    @Override
    public void packagingMajorAllotCommissionRemind1(OrderReportDTO orderReportDTO) {
        List<ReportDTO> reports = orderReportDTO.getReports();
        if (CollectionUtil.isNotEmpty(reports)){
            List<Long> productionIds = reports.stream().map(ReportDTO::getId).collect(Collectors.toList());
            List<MajorProductionAllot> allots = majorProductionAllotService.list(new LambdaQueryWrapper<MajorProductionAllot>().in(MajorProductionAllot::getMajorProductionId, productionIds)
                    .eq(MajorProductionAllot::getUserType, UserTypeEnum.EVALUATE.name()).eq(BaseEntity::getDeleted, Boolean.FALSE));
            if (CollectionUtil.isEmpty(allots)){
                productionIds.stream().forEach(x->{
                    MajorProductionVO majorProductionVO = majorProductionMapper.getMajorProductionById(x);
                    SystemMessageBO systemMessageBO = new SystemMessageBO();
                    systemMessageBO.setReceiver(majorProductionVO.getDepartmentLeaderId());
                    systemMessageBO.setMessage(MAJOR_ALLOT_COMMISSION_CONTENT.replace("reportNo",majorProductionVO.getReportNo()));
                    systemMessageBO.setTitle(MAJOR_ALLOT_COMMISSION_REMIND1.getName());
                    systemMessageBO.setType(MAJOR_ALLOT_COMMISSION_REMIND1.name());
                    systemMessageBO.setParams(JSON.toJSONString(majorProductionVO));
                    systemMessageHandler.sendMessage(systemMessageBO);
                });

            }
        }


    }
}
